/*
** ACEpansion generic library header
**
** Copyright  2020 by Philippe Rimauro
**
** Compatible with:
**  - MorphOS/gcc
**  - AmigaOS/vbcc
*/

#ifndef LIB_HEADER_H
#define LIB_HEADER_H

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif
#ifdef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif
#ifndef LIBRARIES_LOCALE_H
#include <libraries/locale.h>
#endif
#ifndef LIBRARIES_MUI_H
#include <libraries/mui.h>
#endif

/*
** Those are globals provided by the library header
*/
extern struct Library *SysBase;
extern struct Library *LocaleBase;
extern struct Locale *Locale;

#ifdef __GNUC__
#define UNUSED __attribute__((unused))
#else
#define UNUSED
#endif

#ifdef USE_INLINE_API
#define ACEPANSION_API inline static
#include "lib_header.c"
#else
#define ACEPANSION_API
#endif

#ifndef MUIA_String_SpellChecking
#define MUIA_String_SpellChecking TAG_IGNORE
#endif

/* Following macros are partially based on the work of Marian 'MaaG^dA' Guc  2006-2013 */

#ifdef __MORPHOS__

/* DoSuperNew not defined here: it is part of alib */

#define DISPATCHER(ClassNAME) \
    static IPTR ClassNAME##_Dispatcher(VOID); \
    static struct EmulLibEntry GATE_##ClassNAME##_Dispatcher = { TRAP_LIB, 0, (VOID (*)(VOID)) ClassNAME##_Dispatcher }; \
    static IPTR ClassNAME##_Dispatcher(VOID) \
    { \
        struct IClass *cl = (struct IClass *)REG_A0; \
        Object *obj = (Object *)REG_A2; \
        Msg msg = (Msg)REG_A1;

#define DISPATCHER_END \
        return DoSuperMethodA(cl, obj, (Msg)msg); \
    }

#define DISPATCHER_REF(ClassNAME) &GATE_##ClassNAME##_Dispatcher

#elif __amigaos4__

#ifndef INTUITION_CLASSES_H
#include <intuition/classes.h>
#endif

APTR DoSuperNew(struct IClass *cl, Object * obj, ...);

#define DISPATCHER(ClassNAME) \
    static IPTR ClassNAME##Dispatcher(struct IClass *cl, Object *obj, Msg msg) \
    {

#define DISPATCHER_END \
        return DoSuperMethodA(cl, obj, msg); \
    }

#define DISPATCHER_REF(ClassNAME) (APTR)ClassNAME##Dispatcher

#elif __AROS__

#ifndef INTUITION_CLASSES_H
#include <intuition/classes.h>
#endif
#include <aros/asmcall.h>

IPTR DoSuperNew(struct IClass *cl, Object *obj, Tag tag1, ...) __stackparm;

#define DISPATCHER(ClassNAME) \
    AROS_UFH3S(IPTR, ClassNAME##Dispatcher, \
        AROS_UFHA(struct IClass *, cl, A0), \
        AROS_UFHA(Object *, obj, A2), \
        AROS_UFHA(Msg, msg, A1)) \
    { \
        AROS_USERFUNC_INIT

#define DISPATCHER_END \
        return (IPTR)DoSuperMethodA(cl, obj, msg); \
        AROS_USERFUNC_EXIT \
    }

#define DISPATCHER_REF(ClassNAME) ClassNAME##Dispatcher

#elif __amigaos__ /* AmigaOS 68k */

#ifndef INTUITION_CLASSES_H
#include <intuition/classes.h>
#endif

APTR DoSuperNew(struct IClass *cl, APTR obj, ...);

#define DoSuperMethodA(cl, obj, msg) DoSuperMethodA(cl, obj, (Msg)msg)

#define DISPATCHER(ClassNAME) \
    static __saveds ULONG ClassNAME##Dispatcher(__reg("a0") struct IClass *cl, __reg("a2") Object *obj, __reg("a1") Msg msg) \
    {

#define DISPATCHER_END \
        return DoSuperMethodA(cl, obj, (Msg)msg); \
    }

#define DISPATCHER_REF(ClassNAME) (APTR)ClassNAME##Dispatcher

#endif



/****** lib_header/GetString ************************************************

* NAME
*  GetString
*
* SYNOPSIS
*  STRPTR GetString(LONG id)
*
* FUNCTION
*  Retreive a translated string from the library's catalog.
*
* INPUTS
*  id
*    Identifier of the string to retrieve.
*
* RESULT
*  A pointer to the corresponding string translated in the current language, or
*  in English if it was not available.
*
* NOTES
*  A NULL pointer may be returned if the provided identifier is invalid.
*
* SEE ALSO
*
*****************************************************************************
*
*/

STRPTR GetString(LONG id);



/****** lib_header/MakeToolType *********************************************

* NAME
*  MakeToolType
*
* SYNOPSIS
*  STRPTR MakeToolType(APTR pool, CONST_STRPTR key, CONST_STRPTR val)
*
* FUNCTION
*  Allocate and format a tooltype from a key and a string value.
*
* INPUTS
*  pool
*    Pointer to the memory pool to use for allocation.
*  key
*    Pointer to the key string of the tooltype.
*  val
*    Pointer to the string value of the tooltype, or NULL if the tooltype to
*    create do not have a value (i.e. if it is a switch).
*
* RESULT
*  A pointer to string containing the tooltype in the form of "key=val" or "key"
*  (in case val was NULL).
*
* NOTES
*  A NULL pointer may be returned when running out of memory.
*
* SEE ALSO
*  MakeToolTypeNum()
*
*****************************************************************************
*
*/

STRPTR MakeToolType(APTR pool, CONST_STRPTR key, CONST_STRPTR val);



/****** lib_header/MakeToolTypeNum ******************************************

* NAME
*  MakeToolTypeNum
*
* SYNOPSIS
*  STRPTR MakeToolType(APTR pool, CONST_STRPTR key, LONG val)
*
* FUNCTION
*  Allocate and format a tooltype from a key and a numeric value.
*
* INPUTS
*  pool
*    Pointer to the memory pool to use for allocation.
*  key
*    Pointer to the key string of the tooltype.
*  val
*    Numeric value of the tooltype.
*
* RESULT
*  A pointer to string containing the tooltype in the form of "key=val".
*
* NOTES
*
* SEE ALSO
*  MakeToolType()
*
*****************************************************************************
*
*/

STRPTR MakeToolTypeNum(APTR pool, CONST_STRPTR key, LONG val);



/****** lib_header/StrLen ***************************************************

* NAME
*  StrLen
*
* SYNOPSIS
*  ULONG StrLen(CONST_STRPTR str);
*
* FUNCTION
*  Compute the length of a NUL terminated string.
*
* INPUTS
*  str
*    NUL terminated string to compute.
*
* RESULT
*  Length of the string (without counting the NUL terminator).
*
* NOTES
*  StrDup()
*
* SEE ALSO
*
*****************************************************************************
*
*/

ULONG StrLen(CONST_STRPTR str);



/****** lib_header/StrDup ***************************************************

* NAME
*  StrDup
*
* SYNOPSIS
*  STRPTR StrDup(CONST_STRPTR str)
*
* FUNCTION
*  Duplicate a string.
*
* INPUTS
*  str
*    NUL terminated string to duplicate.
*
* RESULT
*  A pointer to the duplicated string (to be freed with FreeVec()).
*
* NOTES
*  A NULL pointer may be returned in case of low memory.
*
* SEE ALSO
*  StrLen()
*
*****************************************************************************
*
*/

STRPTR StrDup(CONST_STRPTR str);

#endif /* LIB_HEADER_H */

